/*
 * Copyright (C) 2001-2006 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.storage;

import mir.config.MirPropertiesConfiguration;
import mir.misc.StringUtil;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.util.GregorianCalendar;
import java.util.Calendar;

public class DatabaseHelper {
  public static final TimeZone INTERNAL_TIMEZONE = TimeZone.getTimeZone(
      MirPropertiesConfiguration.instance().getString("Mir.DefaultTimezone"));
  public static final DateFormat INTERNAL_DATE_FORMAT =
      new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

  private DatabaseHelper() {
  }

  static {
    INTERNAL_DATE_FORMAT.setTimeZone(INTERNAL_TIMEZONE);
  }

  public static String convertDateToInternalRepresenation(Date aDate) {
    Calendar calendar = new GregorianCalendar();
    calendar.setTime(aDate);
    calendar.setTimeZone(INTERNAL_TIMEZONE);
    int offset = calendar.get(Calendar.ZONE_OFFSET);
    boolean positive = true;
    if (offset<0) {
      positive=false;
      offset = -offset;
    }
    int offsethours = offset / (60*60*1000);

    return INTERNAL_DATE_FORMAT.format(aDate) + (positive?"+":"-") +
        StringUtil.zeroPaddingNumber((long) offsethours, 2, 2);
  }

  public static String getInternalTimeZoneOffset(Date aDate) {
    Calendar calendar = new GregorianCalendar();
    calendar.setTime(aDate);
    calendar.setTimeZone(INTERNAL_TIMEZONE);

    int offset = calendar.get(Calendar.DST_OFFSET);
    boolean positive = true;
    if (offset<0) {
      positive=false;
      offset = -offset;
    }
    int offsethours = offset / 60*60*1000;
    int offsetminutes = ( offset / 60*1000 ) % 60;

    return (positive?"+":"-") +
            StringUtil.zeroPaddingNumber((long) offsethours, 2, 2) + ":" +
            StringUtil.zeroPaddingNumber((long) offsetminutes, 2, 2);
  }
}
