package mir.util.xml;

/**
 * Class to encapsulate an XML-style name (either for tagnames,
 * or attribute names).
 *
 * <p>
 * Usage requirements:
 * <ul>
 *   <li> Namespace aware parsers must supply a local name
 *        and a namespace URI
 *   <li> Non-namespace aware parsers should supply a local name,
 *        and, if present and applicable, also a prefix
 * </ul>
 */

public class XMLName {
  private String namespaceURI;
  private String localName;
  private String prefix;

  public XMLName(String aLocalName) {
    this(null, null, aLocalName);
  }

  public XMLName(String aNamespaceURI, String aPrefix, String aLocalName) {
    localName="";
    prefix="";
    namespaceURI="";

    if (aLocalName!=null)
      localName = aLocalName;
    if (aPrefix!=null)
      prefix = aPrefix;
    if (aNamespaceURI!=null)
      namespaceURI = aNamespaceURI;
  }

  public XMLName(String aNamespaceURI, String aLocalName) {
    this (aNamespaceURI, null, aLocalName);
  }

  /**
   * Returns the namespace URI if available
   */
  public String getNamespaceURI() {
    return namespaceURI;
  }

  /**
   * Returns the name of the tag, stripped from prefix
   */
  public String getLocalName() {
    return localName;
  }

  /**
   * Returns the prefix of the tag, if present
   */
  public String getPrefix() {
    return prefix;
  }

  public int hashCode() {
    if (namespaceURI == null)
      return localName.hashCode() + 3*prefix.hashCode();
		return localName.hashCode() + 3*namespaceURI.hashCode();
  }

  public String toString() {
    return ((namespaceURI.length()>0)? "["+namespaceURI+"]":"")+((prefix.length()>0)?prefix+":":"")+localName;
  }

  public boolean equals(Object anObject) {
    if (anObject instanceof mir.util.xml.XMLName) {
      if (namespaceURI==null)
        return (((mir.util.xml.XMLName) anObject).namespaceURI == null) &&
               prefix.equals(((mir.util.xml.XMLName) anObject).prefix) &&
               localName.equals(((mir.util.xml.XMLName) anObject).localName);
      else
        return namespaceURI.equals(((mir.util.xml.XMLName) anObject).namespaceURI) &&
               localName.equals(((mir.util.xml.XMLName) anObject).localName);
    }
		return false;
  }
}
