/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.localizer;

/**
 * <p>Localizers  provide   a  customization  framework   that  allows
 * different mir sites to behave differently. This is effectively used
 * by various  indymedia sites to  customize things such  open posting
 * mechanisms, open posting  validation, data model enhancements, etc.
 * </p>
 * 
 * <p>The   <code>MirLocalizer</code>  interface  defines  the centralized
 * localizer,     that     may  be accessed     via     the     global
 * <function>MirGlobal.localizer()</function>  function.  The  central
 * localizer  provides accessors to  domain specific  localizers (like
 * <code>MirBasicOpenPostingLocalizer</code>).  </p>
 *
 * <p>      Default     behavior      is      provided     by      the
 * <code>MirBasicLocalizer</code>  and it's associated  classes. These
 * classes can be extended to override default behavior.</p>
 *
 */

public interface MirLocalizer {
  public MirProducerLocalizer producers() throws MirLocalizerFailure;
  public MirAdminInterfaceLocalizer adminInterface() throws MirLocalizerFailure;
  public MirOpenPostingLocalizer openPostings() throws MirLocalizerFailure;
  public MirProducerAssistantLocalizer producerAssistant() throws MirLocalizerFailure;
  public MirGeneratorLocalizer generators() throws MirLocalizerFailure;
  public MirDataModelLocalizer dataModel() throws MirLocalizerFailure;
  public MirMediaLocalizer media() throws MirLocalizerFailure;
}