/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.servlet;

import mir.entity.adapter.EntityAdapterEngine;
import mir.servlet.AdminServletModule;
import mir.servlet.ServletModuleExc;
import mir.servlet.ServletModuleFailure;
import mir.util.URLBuilder;
import mircoders.global.MirGlobal;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.Map;

public class ServletModuleAdmin extends AdminServletModule {
  public void defaultAction(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    start(aRequest, aResponse);
  }

  public void superusermenu(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    URLBuilder urlBuilder = new URLBuilder();

    try {
      Map responseData = ServletHelper.makeGenerationData(aRequest, aResponse, getLocales(aRequest));
      urlBuilder.setValue("module", "Admin");
      urlBuilder.setValue("do", "superusermenu");

      responseData.put("thisurl" , urlBuilder.getQuery());

      ServletHelper.generateResponse(aResponse.getWriter(), responseData, "superusermenu.template");
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  /**
   * Servlet handler method to reload the configuration for scripts.
   * The output will be aimed at shell sripts.
   */
  public void reload(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    String result = "reload successfull";

    try {
      MirGlobal.reloadConfigurations();
    }
    catch (Throwable t) {
      result = "error occurred while reloading: " +t.toString();
    }

    try {
      Writer writer = aResponse.getWriter();
      writer.write(result);
    }
    catch (IOException e) {
      throw new ServletModuleFailure(e);
    }
  }

  /**
   * Servlet handler method to reload the configuration.
   */
  public void reloadconfiguration(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    try {
      MirGlobal.reloadConfigurations();
       ServletHelper.generateInfoMessage(aRequest, aResponse, getLocales(aRequest),
           "bundles/admin", "etc/bundles/adminlocal", "reloadSuccessfull", "", "");
    }
    catch (Throwable t) {
      ServletHelper.generateInfoMessage(aRequest, aResponse, getLocales(aRequest),
           "bundles/admin", "etc/bundles/adminlocal", "reloadFailed", t.toString(), "");
    }
  }

  public void start(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    String defaultStartTemplate = getConfiguration().getString("Mir.StartTemplate");

    try {
      Map templateData = ServletHelper.makeGenerationData(aRequest, aResponse, getLocales(aRequest),
          "bundles/admin", "etc/bundles/adminlocal");

      List messages = EntityAdapterEngine.retrieveAdapterList(getModel(),
          "internalMessage", "", "webdb_create desc", 10, 0);

      templateData.put("messages", messages);

      templateData.put("fileeditentries",
          ServletHelper.getServletModuleFileEdit().getEntries());
      templateData.put("administeroperations",
          ServletHelper.getServletModuleLocalizer().getAdministerOperations());

      templateData.put("searchvalue", null);
      templateData.put("searchfield", null);
      templateData.put("searchispublished", null);
      templateData.put("searcharticletype", null);
      templateData.put("searchorder", null);
      templateData.put("searchtopic", null);
      templateData.put("selectarticleurl", null);
      templateData.put("recipes", MirGlobal.localizer().producers().getRecipeNames());

      String startTemplate = MirGlobal.localizer().adminInterface().getAdminPageGenerator(
          "admin.start", templateData,
          ServletHelper.getUserAdapter(aRequest), defaultStartTemplate);

      ServletHelper.generateResponse(aResponse.getWriter(), templateData, startTemplate);
    }
    catch (Exception e) {
      throw new ServletModuleFailure(e);
    }
  }

}
